@@ -92,7 +92,7 @@ class AsyncResultCollection:
             async_result.result_object.progress * 100
             for async_result in self._async_results
         ]
-        new_progress = float(sum(new_progress)) / len(new_progress)
+        new_progress = int(sum(new_progress) / len(new_progress))
 
         delta = new_progress - self._current_progress
         self._current_progress += delta
@@ -126,6 +126,7 @@ class AsyncResultCollection:
     def finalize(self):
         """Reports on final status."""
 
+        # List of (identifier string, exception message) tuples for each failure.
         failures = []
 
         for async_result in self._async_results:
@@ -137,14 +138,19 @@ class AsyncResultCollection:
             )
 
             if not async_result.result_object.is_successful:
-                failures.append(async_result)
+
+                identifier = (
+                    f"{async_result.result_object.name} "
+                    f"({async_result.result_object.action_id}): "
+                )
 
                 if async_result.result_object.job_errors:
                     LOGGER.error(
                         "Action %s failed: %s",
                         async_result.result_object.action_id,
-                        async_result.result_object.job_errors,
+                        async_result.result_object.job_errors.exception,
                     )
+                    exception = f"{async_result.result_object.job_errors.exception}"
 
                 elif async_result.result_object.status == "FAILURE":
                     LOGGER.error(
@@ -153,8 +159,18 @@ class AsyncResultCollection:
                         "available).",
                         async_result.result_object.action_id,
                     )
+                    exception = "Unknown failure"
 
-        if failures:
-            raise RuntimeError(
-                f"Action failures: {len(failures)} of {len(self._async_results)}"
-            )
+                failures.append((identifier, exception))
+
+        if not failures:
+            return
+
+        if len(self._async_results) == 1:
+            raise RuntimeError(failures[0][1])
+
+        failure_message = "\n".join("".join(failure) for failure in failures)
+        raise RuntimeError(
+            f"Action failures: {len(failures)} of {len(self._async_results)}\n"
+            f"{failure_message}"
+        )
